@@ -34,7 +34,7 @@
 # })
 # circos.clear()
 #
-# \dontrun{
+# if(FALSE) {
 # # NOTE: following takes quite a long time to run
 # load(system.file("extdata", "doodle.RData", package = "circlize"))
 # circos.par("cell.padding" = c(0, 0, 0, 0))
@@ -51,10 +51,13 @@
 # }, track.height = 0.25, bg.border = NA)
 # circos.clear()
 # }
-circos.raster = function(image, x, y, width, height, 
+circos.raster = function(
+	image, x, y, 
+	width, height, 
 	facing = c("inside", "outside", "reverse.clockwise", "clockwise",
         "downward", "bending.inside", "bending.outside"),
-    niceFacing = FALSE, sector.index = get.cell.meta.data("sector.index"), 
+    niceFacing = FALSE, 
+    sector.index = get.cell.meta.data("sector.index"), 
     track.index = get.cell.meta.data("track.index"), 
     scaling = 1) {
 
